"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Carbone = void 0;
const n8n_workflow_1 = require("n8n-workflow");
const GenericFunctions_1 = require("./GenericFunctions");
class Carbone {
    constructor() {
        this.description = {
            displayName: 'Carbone',
            name: 'carbone',
            icon: 'file:carbone.svg',
            group: ['transform'],
            version: 1,
            description: 'Node for the Carbone API',
            defaults: {
                name: 'carbone',
            },
            inputs: ['main'],
            outputs: ['main'],
            credentials: [
                {
                    name: 'carboneApi',
                    required: true,
                },
            ],
            properties: [
                {
                    displayName: 'Operation',
                    name: 'operation',
                    type: 'options',
                    noDataExpression: true,
                    options: [
                        {
                            name: 'Add Template',
                            value: 'addTemplate',
                            description: 'Add new template',
                            action: 'Add new template',
                        },
                        {
                            name: 'Delete Template',
                            value: 'deleteTemplate',
                            description: 'Delete Template By ID',
                            action: 'Delete template by id',
                        },
                        {
                            name: 'Download Rendered Report',
                            value: 'downloadReport',
                            description: 'Download the rendered report Using the reports ID',
                            action: 'Download the rendered report using the reports id',
                        },
                        {
                            name: 'Get Template',
                            value: 'getTemplate',
                            description: 'Get Template By ID',
                            action: 'Get template by id',
                        },
                        {
                            name: 'Render Report',
                            value: 'renderReport',
                            description: 'Render reports',
                            action: 'Render reports',
                        },
                    ],
                    default: 'getTemplate',
                },
                {
                    displayName: 'ID',
                    name: 'id',
                    type: 'string',
                    description: 'ID of resource to Get, Edit, Delete or add data to',
                    default: '',
                    displayOptions: {
                        hide: {
                            operation: [
                                'addTemplate',
                            ],
                        },
                    },
                },
                {
                    displayName: 'Body Type',
                    name: 'bodyType',
                    type: 'options',
                    options: [
                        {
                            name: 'Json',
                            value: 'json',
                        },
                        {
                            name: 'Per Field',
                            value: 'perField',
                        },
                    ],
                    displayOptions: {
                        show: {
                            operation: [
                                'renderReport',
                            ],
                        },
                    },
                    default: 'json',
                    description: 'Data for template to render the report',
                },
                {
                    displayName: 'Body',
                    name: 'body',
                    type: 'string',
                    description: 'JSON Body containing report data',
                    default: '',
                    displayOptions: {
                        show: {
                            operation: [
                                'renderReport',
                            ],
                            bodyType: [
                                'json',
                            ],
                        },
                    },
                    typeOptions: {
                        rows: 10,
                    },
                },
                {
                    displayName: 'Binary Property',
                    name: 'binaryPropertyName',
                    type: 'string',
                    displayOptions: {
                        show: {
                            operation: [
                                'addTemplate',
                                'getTemplate',
                                'downloadReport',
                            ],
                        },
                    },
                    default: 'data',
                    description: 'Name of the binary property which contains the data for the file to be uploaded. For Form-Data Multipart, they can be provided in the format: <code>"sendKey1:binaryProperty1,sendKey2:binaryProperty2</code>',
                },
                {
                    displayName: 'Field Data',
                    name: 'data',
                    placeholder: 'Add field data',
                    type: 'fixedCollection',
                    typeOptions: {
                        multipleValues: true,
                        sortable: true,
                    },
                    default: {},
                    displayOptions: {
                        show: {
                            operation: [
                                'renderReport',
                            ],
                            bodyType: [
                                'perField',
                            ],
                        },
                    },
                    options: [
                        {
                            name: 'field',
                            displayName: 'Field',
                            values: [
                                {
                                    displayName: 'Field Name',
                                    name: 'fieldName',
                                    type: 'string',
                                    default: '',
                                    description: 'Field name to include in item',
                                },
                                {
                                    displayName: 'Field Type',
                                    name: 'fieldType',
                                    type: 'options',
                                    default: 'string',
                                    description: 'Value for the field',
                                    options: [
                                        {
                                            name: 'String',
                                            value: 'string',
                                        },
                                        {
                                            name: 'Array',
                                            value: 'array',
                                        },
                                    ],
                                },
                                {
                                    displayName: 'Field Value',
                                    name: 'fieldValue',
                                    type: 'string',
                                    default: '',
                                    description: 'Value for the field',
                                },
                            ],
                        },
                    ],
                },
            ],
        };
    }
    async execute() {
        const items = this.getInputData();
        const returnItems = [];
        let item;
        const operation = this.getNodeParameter('operation', 0, '');
        for (let itemIndex = 0; itemIndex < items.length; itemIndex++) {
            try {
                item = items[itemIndex];
                if (operation === "addTemplate") {
                    const endpoint = "template";
                    const qs = {};
                    const body = {};
                    if (item.binary === undefined) {
                        throw new n8n_workflow_1.NodeOperationError(this.getNode(), 'No binary data exists on item!');
                    }
                    const binaryPropertyNameFull = this.getNodeParameter('binaryPropertyName', itemIndex);
                    const binaryPropertyNames = binaryPropertyNameFull.split(',').map(key => key.trim());
                    for (const propertyData of binaryPropertyNames) {
                        const propertyName = 'template';
                        const binaryPropertyName = propertyData;
                        if (item.binary[binaryPropertyName] === undefined) {
                            throw new n8n_workflow_1.NodeOperationError(this.getNode(), `No binary data property "${binaryPropertyName}" does not exists on item!`);
                        }
                        const binaryProperty = item.binary[binaryPropertyName];
                        const binaryDataBuffer = await this.helpers.getBinaryDataBuffer(itemIndex, binaryPropertyName);
                        body[propertyName] = {
                            value: binaryDataBuffer,
                            options: {
                                filename: binaryProperty.fileName,
                                contentType: binaryProperty.mimeType,
                            },
                        };
                    }
                    const newItem = {
                        json: {},
                        binary: {},
                    };
                    const data = await GenericFunctions_1.carboneFileUploadRequest.call(this, 'Post', endpoint, body, qs);
                    newItem.json = data.data;
                    returnItems.push(newItem);
                }
                if (operation === "deleteTemplate") {
                    const templateId = this.getNodeParameter('id', itemIndex, '');
                    const endpoint = `template/${templateId}`;
                    const newItem = {
                        json: {},
                        binary: {},
                    };
                    newItem.json = await GenericFunctions_1.carboneApiRequest.call(this, 'Delete', endpoint);
                    returnItems.push(newItem);
                }
                if (operation === "downloadReport") {
                    const reportId = this.getNodeParameter('id', itemIndex, '');
                    const endpoint = `render/${reportId}`;
                    const binaryPropertyName = this.getNodeParameter('binaryPropertyName', itemIndex);
                    const newItem = {
                        json: {},
                        binary: {},
                    };
                    if (items[itemIndex].binary !== undefined) {
                        newItem.binary = items[itemIndex].binary;
                    }
                    newItem.json = items[itemIndex].json;
                    const response = await GenericFunctions_1.carboneFileDownloadRequest.call(this, 'Get', endpoint);
                    newItem.binary[binaryPropertyName] = await this.helpers.prepareBinaryData(response.body, "Report");
                    returnItems.push(newItem);
                }
                if (operation === "getTemplate") {
                    const templateId = this.getNodeParameter('id', itemIndex, '');
                    const endpoint = `template/${templateId}`;
                    const binaryPropertyName = this.getNodeParameter('binaryPropertyName', itemIndex);
                    const newItem = {
                        json: {},
                        binary: {},
                    };
                    if (items[itemIndex].binary !== undefined) {
                        newItem.binary = items[itemIndex].binary;
                    }
                    newItem.json = items[itemIndex].json;
                    const response = await GenericFunctions_1.carboneFileDownloadRequest.call(this, 'Get', endpoint);
                    newItem.binary[binaryPropertyName] = await this.helpers.prepareBinaryData(response.body, "Template");
                    returnItems.push(newItem);
                }
                if (operation === "renderReport") {
                    const templateId = this.getNodeParameter('id', itemIndex, '');
                    const endpoint = `render/${templateId}`;
                    const bodyType = this.getNodeParameter('bodyType', itemIndex, '');
                    const body = {};
                    if (bodyType === 'json') {
                        const tempBody = this.getNodeParameter('body', itemIndex, '');
                        try {
                            body.data = JSON.parse(tempBody);
                        }
                        catch (error) {
                            throw new n8n_workflow_1.NodeOperationError(this.getNode(), error, {
                                itemIndex,
                            });
                        }
                    }
                    if (bodyType === 'perField') {
                        const fieldArray = this.getNodeParameter('data.field', itemIndex, '');
                        const data = {};
                        for (let fieldIndex = 0; fieldIndex < fieldArray.length; fieldIndex++) {
                            const field = fieldArray[fieldIndex];
                            if (field.fieldType === 'string') {
                                data[field.fieldName] = field.fieldValue;
                            }
                            if (field.fieldType === 'array') {
                                data[field.fieldName] = JSON.parse(field.fieldValue);
                            }
                        }
                        body.data = data;
                    }
                    const newItem = {
                        json: {},
                        binary: {},
                    };
                    body.convertTo = 'pdf';
                    newItem.json = await GenericFunctions_1.carboneApiRequest.call(this, 'Post', endpoint, body);
                    returnItems.push(newItem);
                }
            }
            catch (error) {
                if (this.continueOnFail()) {
                    items.push({ json: this.getInputData(itemIndex)[0].json, error, pairedItem: itemIndex });
                }
                else {
                    if (error.context) {
                        error.context.itemIndex = itemIndex;
                        throw error;
                    }
                    throw new n8n_workflow_1.NodeOperationError(this.getNode(), error, {
                        itemIndex,
                    });
                }
            }
        }
        return this.prepareOutputData(returnItems);
    }
}
exports.Carbone = Carbone;
//# sourceMappingURL=Carbone.node.js.map